/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.paimon.data;

import org.apache.paimon.memory.MemorySegment;
import org.apache.paimon.utils.MathUtils;

import java.util.ArrayList;

/** Segments contains multiple {@link MultiSegments}. */
public interface Segments {

    long totalMemorySize();

    static Segments create(ArrayList<MemorySegment> segments, int limitInLastSegment) {
        if (segments.size() == 1) {
            MemorySegment segment = segments.get(0);
            int roundUp = MathUtils.roundUpToPowerOf2(limitInLastSegment);
            if (roundUp < segment.size()) {
                MemorySegment newSegment = MemorySegment.allocateHeapMemory(roundUp);
                segment.copyTo(0, newSegment, 0, limitInLastSegment);
                segment = newSegment;
            }
            return new SingleSegments(segment, limitInLastSegment);
        } else {
            return new MultiSegments(segments, limitInLastSegment);
        }
    }
}
